"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findLastSavedMessage = exports.SnapshotBuilder = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_stream_1 = require("node:stream");
const web_1 = require("node:stream/web");
const common_1 = require("../common");
const dayjs_1 = tslib_1.__importDefault(require("dayjs"));
const utc_1 = tslib_1.__importDefault(require("dayjs/plugin/utc"));
const config_1 = require("../config");
dayjs_1.default.extend(utc_1.default);
function accountToDir(account) {
    return account.replace('@', '_at_');
}
class SnapshotBuilder {
    constructor() {
        this.now = dayjs_1.default.utc().format('YYYY-MM-DD_HH-mm-ss-SSS');
    }
    async getSnapshotWriter(account, onMessageSave) {
        const accountDir = accountToDir(account);
        const dir = getSnapshotPath(this.now, accountDir);
        await node_fs_1.default.promises.mkdir(dir, { recursive: true });
        const messageWithAttachments = await this.prepareMessageWithAttachmentsWritable(accountDir, onMessageSave);
        const writeLabels = async (labels) => {
            await node_fs_1.default.promises.writeFile(getLabelsPath(this.now, accountDir), labels.map(label => JSON.stringify(label)).join('\n'));
        };
        return { messageWithAttachments, writeLabels };
    }
    async prepareMessageWithAttachmentsWritable(accountDir, onMessageSave) {
        const fileStream = node_stream_1.Writable.toWeb(node_fs_1.default.createWriteStream(getMessagesPath(this.now, accountDir)));
        const attachmentsFile = await node_fs_1.default.promises.open(getAttachmentsPath(this.now, accountDir), 'w');
        let savedMessages = 0;
        const saveAttachments = new web_1.TransformStream({
            transform: async ({ message, attachments }, controller) => {
                if (attachments.length > 0) {
                    const attachmentsNDJSON = attachments.map(attachment => `${JSON.stringify(attachment)}\n`).join('');
                    await attachmentsFile.writeFile(attachmentsNDJSON);
                }
                controller.enqueue(message);
            },
            flush: () => attachmentsFile.close(),
        }, common_1.QUEUING_STRATEGY, common_1.QUEUING_STRATEGY);
        const ndjsonStream = new web_1.TransformStream({
            transform: (message, controller) => {
                const line = JSON.stringify(message);
                controller.enqueue(`${line}\n`);
                savedMessages++;
                onMessageSave === null || onMessageSave === void 0 ? void 0 : onMessageSave(savedMessages);
            },
        }, common_1.QUEUING_STRATEGY, common_1.QUEUING_STRATEGY);
        saveAttachments.readable
            .pipeThrough(ndjsonStream)
            .pipeTo(fileStream)
            .then();
        return saveAttachments.writable;
    }
}
exports.SnapshotBuilder = SnapshotBuilder;
async function findLastSavedMessage(account) {
    const snapshots = await node_fs_1.default.promises.readdir((0, config_1.getConfig)().snapshotsPath).catch(error => {
        if (error.code === 'ENOENT') {
            return [];
        }
        throw error;
    });
    for (const snapshot of snapshots.sort().reverse()) {
        try {
            const lastLine = await (0, common_1.readLastLine)(getMessagesPath(snapshot, account));
            if (lastLine) {
                return JSON.parse(lastLine);
            }
        }
        catch (error) {
            if (error.code !== 'ENOENT') {
                throw error;
            }
        }
    }
    return undefined;
}
exports.findLastSavedMessage = findLastSavedMessage;
function getSnapshotPath(snapshot, account) {
    return `${(0, config_1.getConfig)().snapshotsPath}/${snapshot}/${accountToDir(account)}`;
}
function getMessagesPath(snapshot, account) {
    return `${getSnapshotPath(snapshot, account)}/messages.ndjson`;
}
function getAttachmentsPath(snapshot, account) {
    return `${getSnapshotPath(snapshot, account)}/attachments.ndjson`;
}
function getLabelsPath(snapshot, account) {
    return `${getSnapshotPath(snapshot, account)}/labels.ndjson`;
}
